const { src, dest, watch, series, parallel } = require('gulp');
const sass = require('gulp-sass')(require('sass'));
const connect = require('gulp-connect');
const postcss = require('gulp-postcss');
const px2rem = require('postcss-px2rem');
const cssnano = require('gulp-cssnano');
const uglify = require('gulp-uglify');
const rename = require('gulp-rename');

// 将屏幕宽度分成10rem:
// 若设计稿宽度375,则remUnit为37.5; 同理750的设计稿remUnit为75;
const remUnit = 37.5;
const processors = [px2rem({ remUnit: remUnit })];

// 处理sass文件
function styles() {
    return src('./src/style/*.scss')
        // 处理sass
        .pipe(sass())
        // px2rem
        .pipe(postcss(processors))
        // 生成css文件到目标目录
        // .pipe(dest('./static/css'))
        // 压缩css
        .pipe(cssnano())
        // 重命名为.min
        .pipe(rename({
            suffix: ".min",
        }))
        // 生成css.min文件到目标目录
        .pipe(dest('./static/css'))
}

// 处理js文件
function scripts() {
    return src('./src/script/*.js')
        // uglify
        .pipe(uglify())
        // 重命名为.min
        .pipe(rename({
            suffix: ".min",
        }))
        // 生成js文件到目标目录
        .pipe(dest('./static/js'))
}

// 开发环境服务
function devServer(cb) {
    connect.server({
        root: './',
        port: 18081,
        livereload: true,
        livereload: {
            port: 58081,
        }
    }, cb);
}

// 监听文件
function watchFiles() {
    watch('src/style/*.scss', styles);
    watch('src/script/*.js', scripts);
    watch(['*.html', 'static/css/*.css', 'static/js/*.js'], function () {
        return src('*')
            .pipe(connect.reload())
    });
}

const build = parallel(
    styles,
    scripts,
);

const serve = parallel(
    watchFiles,
    devServer,
);

/**
 * 编译构建：gulp build
 * 启动服务：gulp serve
 * 默认(启动服务)：gulp
 */
exports.build = build;
exports.serve = serve;
exports.default = serve;
